package com.io.yutian.elementoriginlib.command.list;

import com.io.yutian.elementoriginlib.command.CommandContext;
import com.io.yutian.elementoriginlib.command.CommandEntity;
import com.io.yutian.elementoriginlib.command.CommandEntry;
import com.io.yutian.elementoriginlib.command.interfaces.Command;
import com.io.yutian.elementoriginlib.command.interfaces.Parameter;
import com.io.yutian.elementoriginlib.command.interfaces.SubCommand;
import com.io.yutian.elementoriginlib.lang.Lang;
import com.io.yutian.elementoriginlib.list.PageList;
import com.io.yutian.elementoriginlib.manager.CommandManager;
import com.io.yutian.elementoriginlib.util.ComponentBuilder;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.event.ClickEvent;
import net.kyori.adventure.text.event.HoverEvent;
import org.bukkit.command.CommandSender;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Command("help")
public class CommandHelp {

    private final static int MAX_PAGE_SIZE = 8;

    private final CommandManager commandManager;
    private final String alias;


    public CommandHelp(CommandManager commandManager, String alias) {
        this.commandManager = commandManager;
        this.alias = alias;
    }

    @SubCommand(nodal = true)
    public void help(CommandContext commandContext, @Parameter(required = false, defaultValue = "1") int page) {
        String commandAlias = alias != null ? alias : commandContext.getLabel();
        CommandSender sender = commandContext.getSender();
        if (page <= 0) {
            sender.sendMessage(Lang.get("command.help.page-error"));
            return;
        }
        List<CommandEntity> commands = commandManager.getCommandEntities();
        Stream<CommandEntity> stream = commands.stream().filter((c) -> c.hasPermission(sender) && c.canInvoke(sender));
        List<CommandEntity> list = stream.collect(Collectors.toList());
        PageList<CommandEntity> pageList = new PageList<>(list, MAX_PAGE_SIZE);
        if (page > pageList.size()) {
            sender.sendMessage(Lang.get("command.help.page-error"));
            return;
        }
        sender.sendMessage(" ");
        List<CommandEntity> commandList = pageList.getList(page);
        sender.sendMessage("§7======[ §e§l"+commandManager.getName()+" §7]======");
        for (CommandEntity command : commandList) {
            StringBuilder stringBuilder = getCommandInfo(command, commandAlias);
            sender.sendMessage(stringBuilder.toString());
        }
        ComponentBuilder componentBuilder = new ComponentBuilder();
        boolean hasUpPage = page > 1;
        boolean hasNextPage = page < pageList.size();
        componentBuilder.add("§7=====");
        if (hasUpPage) {
            componentBuilder.add(" §7["+getColor(true)+"◀§7] ", ClickEvent.clickEvent(ClickEvent.Action.RUN_COMMAND, "/"+commandAlias+" help "+(page-1)), HoverEvent.hoverEvent(HoverEvent.Action.SHOW_TEXT, Component.text("§f上一页")));
        } else {
            componentBuilder.add(" §7["+getColor(false)+"◀§7] ");
        }
        componentBuilder.add("§7====");
        componentBuilder.add("(§a"+page+"§f/§e"+pageList.size()+"§7)");
        componentBuilder.add("§7====");
        if (hasNextPage) {
            componentBuilder.add(" §7["+getColor(true)+"▶§7] ", ClickEvent.clickEvent(ClickEvent.Action.RUN_COMMAND, "/"+commandAlias+" help "+(page+1)), HoverEvent.hoverEvent(HoverEvent.Action.SHOW_TEXT, Component.text("§f下一页")));
        } else {
            componentBuilder.add(" §7["+getColor(false)+"▶§7] ");
        }
        componentBuilder.add("§7=====");
        sender.sendMessage(componentBuilder.build());
    }

    private StringBuilder getCommandInfo(CommandEntity command, String commandAlias) {
        StringBuilder stringBuilder = new StringBuilder("§6/"+ commandAlias +" "+ command.getCommand());
        stringBuilder.append("§f");
        System.out.println(commandAlias);
        if (command.getChildrens().size() > 0) {

        } else {

        }
        Optional<String> optional = Lang.getOptional("command."+command.getCommand()+".description");
        if (optional.isPresent()) {
            stringBuilder.append(" ");
            stringBuilder.append("§7- §f"+ optional.get());
        }
//        if (command.getChildrens().size() > 0) {
//            StringBuilder sb = new StringBuilder();
//            sb.append(" [");
//            int i = 0;
//            for (CommandNode node : command.getCommandNodes()) {
//                sb.append(node.getName());
//                if (i + 1 < command.getCommandNodes().size()) {
//                    sb.append("/");
//                }
//                i++;
//            }
//            sb.append("]");
//            stringBuilder.append(sb);
//        } else {
//            for (Argument argument : command.getArguments()) {
//                stringBuilder.append(" ");
//                stringBuilder.append("<"+argument.getName()+">");
//            }
//        }
//        Optional<String> optional = Lang.getOptional("command."+command.getName()+".description");
//        if (optional.isPresent()) {
//            stringBuilder.append(" ");
//            stringBuilder.append("§7- §f"+ optional.get());
//        }
        return stringBuilder;
    }

    private String getColor(boolean hasPage) {
        return hasPage ? "§a" : "§c";
    }

}
