package com.io.yutian.elementoriginlib.item;

import com.io.yutian.elementoriginlib.item.stat.ItemStat;
import com.io.yutian.elementoriginlib.item.stat.ItemStats;
import com.io.yutian.elementoriginlib.item.stat.StatData;
import com.io.yutian.elementoriginlib.tag.ItemProxy;
import org.bukkit.inventory.ItemStack;
import org.jetbrains.annotations.NotNull;

import java.util.*;

public class OriginItem {

    private ItemStack itemStack;
    private ItemProxy itemProxy;
    private Map<ItemStat, StatData> stats = new HashMap<>();

    private OriginItemStackBuilder builder;

    public OriginItem(ItemStack itemStack) {
        this.itemStack = itemStack;
        this.itemProxy = new ItemProxy(itemStack);

        this.builder = new OriginItemStackBuilder(this);
    }

    public void setStatData(@NotNull ItemStat stat, @NotNull StatData data) {
        this.stats.put(stat, data);
        this.itemStack = builder().build();
    }

    public void removeStatData(@NotNull ItemStat stat) {
        this.stats.remove(stat);
        this.itemStack = builder().build();
    }

    public <S extends StatData> S getStatData(@NotNull ItemStat<S> stat) {
        if (!hasStatData(stat)) {
            stat.load(this);
        }
        return (S) this.stats.get(stat);
    }

    public boolean hasStatData(@NotNull ItemStat stat) {
        if (!this.stats.containsKey(stat)) {
            try {
                stat.load(this);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return this.stats.containsKey(stat);
    }

    @NotNull
    public OriginItemStackBuilder builder() {
        return this.builder;
    }

    @NotNull
    public Set<ItemStat> getStatTypes() {
        return Collections.unmodifiableSet(this.stats.keySet());
    }

    @NotNull
    public Map<ItemStat, StatData> getStats() {
        for (ItemStat stat : ItemStats.getItemStats()) {
            if (!hasStatData(stat)) {
                try {
                    stat.load(this);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
        return Collections.unmodifiableMap(stats);
    }

    public ItemStack getItemStack() {
        return itemStack;
    }

    public ItemProxy getItemProxy() {
        return itemProxy;
    }

    @Override
    public OriginItem clone() {
        OriginItem originItem = new OriginItem(itemStack.clone());
        Iterator iterator = this.stats.keySet().iterator();
        while(iterator.hasNext()) {
            ItemStat itemStat = (ItemStat) iterator.next();
            originItem.stats.put(itemStat, this.stats.get(itemStat));
        }
        return originItem;
    }

}
