package com.io.yutian.elementoriginlib.item;

import com.io.yutian.elementoriginlib.item.stat.StatData;
import com.io.yutian.elementoriginlib.tag.ItemProxy;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

public class OriginItemStackBuilder {

    private final OriginItem originItem;

    protected ItemStack itemStack;
    protected ItemMeta itemMeta;

    protected ItemProxy itemProxy;

    public OriginItemStackBuilder(OriginItem originItem) {
        this.originItem = originItem;
        this.itemStack = originItem.getItemStack();
        this.itemMeta = itemStack.getItemMeta();
        this.itemProxy = originItem.getItemProxy();
    }

    @NotNull
    public ItemStack build() {
        buildCompounds();
        return itemStack;
    }

    @NotNull
    public ItemMeta getItemMeta() {
        return itemMeta;
    }

    private void buildCompounds() {
        originItem.getStats().forEach((stat, statData) -> stat.whenApplied(this, statData));
        itemStack = itemProxy.getItemStack();
        itemMeta = itemStack.getItemMeta();
        originItem.getStats().forEach((stat, statData) -> stat.applyMeta(this, statData));
        itemStack.setItemMeta(itemMeta);
        List<String> lores = itemMeta.hasLore() ? itemMeta.getLore() : new ArrayList<>();
        originItem.getStats().forEach((stat, statData) -> stat.whenApplyLore(this, statData, lores));
        itemStack.setLore(lores);
    }

    @NotNull
    public ItemProxy getNBTItem() {
        return itemProxy;
    }

    public OriginItem getOriginItem() {
        return originItem;
    }

}
