package com.io.yutian.elementoriginlib.item.stat;

import com.io.yutian.elementoriginlib.item.stat.list.IdStat;
import com.io.yutian.elementoriginlib.logger.Logger;
import org.jetbrains.annotations.Nullable;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

public class ItemStats {

    private static final Logger LOGGER = Logger.getLogger(ItemStats.class);

    private static Map<Class<? extends ItemStat>, ItemStat<?>> itemStats = new HashMap<>();

    static {
        register(IdStat.class, new IdStat());
    }

    public static <I extends ItemStat> void register(Class<I> clazz, I itemStat) {
        if (itemStats.containsKey(itemStat.getId())) {
            LOGGER.warn("ItemStat "+clazz.getName()+" 已经注册过了");
            return;
        }
        itemStats.put(clazz, itemStat);
    }

    public static void unregister(ItemStat itemStat) {
        itemStats.remove(itemStat.getId());
    }

    @Nullable
    public static ItemStat getItemStat(Class<? extends ItemStat> clazz) {
        return itemStats.get(clazz);
    }

    @Nullable
    public static ItemStat getItemStat(String id) {
        for (ItemStat itemStat : itemStats.values()) {
            if (itemStat.getId().equalsIgnoreCase(id)) {
                return itemStat;
            }
        }
        return null;
    }

    public static Collection<ItemStat> getItemStats() {
        return Collections.unmodifiableCollection(itemStats.values());
    }

}
