package com.io.yutian.elementoriginlib.tag;

import net.minecraft.nbt.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public interface ITag {

    Object getValue();

    byte getTypeId();

    private static String getTag(byte typeByte) {
        return switch (typeByte) {
            case 1 -> "TagByte";
            case 2 -> "TagShort";
            case 3 -> "TagInt";
            case 4 -> "TagLong";
            case 5 -> "TagFloat";
            case 6 -> "TagDouble";
            case 7 -> "TagByteArray";
            case 8 -> "TagString";
            case 9 -> "TagList";
            case 10 -> "TagCompound";
            case 11 -> "TagIntArray";
            case 12 -> "TagLongArray";
            default -> null;
        };
    }

    static ITag asObject(Object object) {
        if (object instanceof Byte b) {
            return new TagByte(b);
        } else if (object instanceof Short value) {
            return new TagShort(value);
        } else if (object instanceof Integer value) {
            return new TagInt(value);
        } else if (object instanceof Long value) {
            return new TagLong(value);
        } else if (object instanceof Float value) {
            return new TagFloat(value);
        } else if (object instanceof Double value) {
            return new TagDouble(value);
        } else if (object instanceof byte[] value) {
            return new TagByteArray(value);
        } else if (object instanceof String value) {
            return new TagString(value);
        } else if (object instanceof int[] value) {
            return new TagIntArray(value);
        } else if (object instanceof long[] value) {
            return new TagLongArray(value);
        } else if (object instanceof Boolean b) {
            return new TagByte((byte) (b ? 1 : 0));
        }
        return null;
    }

    static net.minecraft.nbt.Tag asNMS(ITag nbt) {
        byte type = nbt.getTypeId();
        switch (type) {
            case 1 -> {
                TagByte tagByte = (TagByte) nbt;
                return ByteTag.valueOf(tagByte.getByte());
            }
            case 2 -> {
                TagShort nbtShort = (TagShort) nbt;
                return ShortTag.valueOf(nbtShort.getShort());
            }
            case 3 -> {
                TagInt tagInt = (TagInt) nbt;
                return IntTag.valueOf(tagInt.getInt());
            }
            case 4 -> {
                TagLong tagLong = (TagLong) nbt;
                return LongTag.valueOf(tagLong.getLong());
            }
            case 5 -> {
                TagFloat nbtfloat = (TagFloat) nbt;
                return FloatTag.valueOf(nbtfloat.getFloat());
            }
            case 6 -> {
                TagDouble tagDouble = (TagDouble) nbt;
                return DoubleTag.valueOf(tagDouble.getDouble());
            }
            case 7 -> {
                TagByteArray tagByteArray = (TagByteArray) nbt;
                return new ByteArrayTag(tagByteArray.getByteArray());
            }
            case 8 -> {
                TagString tagString = (TagString) nbt;
                return StringTag.valueOf(tagString.getString());
            }
            case 9 -> {
                TagList nbtTagList = (TagList) nbt;
                List<net.minecraft.nbt.Tag> list = new ArrayList<>();
                for (Object base : nbtTagList.getList()) {
                    list.add(asNMS((ITag) base));
                }
                ListTag nbtTagList1 = new ListTag();
                for (net.minecraft.nbt.Tag nbt1 : list) {
                    nbtTagList1.add(nbt1);
                }
                return nbtTagList1;
            }
            case 10 -> {
                TagCompound tagCompound = (TagCompound) nbt;
                CompoundTag nbtTagCompound = new CompoundTag();
                for (String key : tagCompound.keySet()) {
                    ITag nbt1 = tagCompound.get(key);
                    nbtTagCompound.put(key, asNMS(nbt1));
                }
                return nbtTagCompound;
            }
            case 11 -> {
                TagIntArray tagIntArray = (TagIntArray) nbt;
                return new IntArrayTag(tagIntArray.getIntArray());
            }
            case 12 -> {
                TagLongArray tagLongArray = (TagLongArray) nbt;
                return new LongArrayTag(tagLongArray.getLongArray());
            }
        }
        return null;
    }

    static ITag as(net.minecraft.nbt.Tag tag) {
        byte type = tag.getId();
        switch (type) {
            case 1:
                ByteTag nbtTagByte = (ByteTag) tag;
                return new TagByte(nbtTagByte.getAsByte());
            case 2:
                ShortTag nbtTagShort = (ShortTag) tag;
                return new TagShort(nbtTagShort.getAsShort());
            case 3:
                IntTag nbtTagInt = (IntTag) tag;
                return new TagInt(nbtTagInt.getAsInt());
            case 4:
                LongTag nbtTagLong = (LongTag) tag;
                return new TagLong(nbtTagLong.getAsLong());
            case 5:
                FloatTag nbtTagFloat = (FloatTag) tag;
                return new TagFloat(nbtTagFloat.getAsFloat());
            case 6:
                DoubleTag nbtTagDouble = (DoubleTag) tag;
                return new TagDouble(nbtTagDouble.getAsDouble());
            case 7:
                ByteArrayTag tagByteArray = (ByteArrayTag) tag;
                return new TagByteArray(tagByteArray.getAsByteArray());
            case 8:
                StringTag nbtTagString = (StringTag) tag;
                return new TagString(nbtTagString.getAsString());
            case 9:
                ListTag nbtTagList = (ListTag) tag;
                List<ITag> list = new ArrayList<>();
                for (net.minecraft.nbt.Tag base : nbtTagList) {
                    list.add(as(base));
                }
                return new TagList(list);
            case 10:
                CompoundTag nbtTagCompound = (CompoundTag) tag;
                Map<String, ITag> map = new HashMap<>();
                for (String key : nbtTagCompound.getAllKeys()) {
                    map.put(key, as(nbtTagCompound.get(key)));
                }
                return new TagCompound(map);
            case 11:
                IntArrayTag nbtTagIntArray = (IntArrayTag) tag;
                return new TagIntArray(nbtTagIntArray.getAsIntArray());
            case 12:
                LongArrayTag nbtTagLongArray = (LongArrayTag) tag;
                return new TagLongArray(nbtTagLongArray.getAsLongArray());
        }
        return null;
    }

}
